/**
 *
 */
package org.quintet.backbeat.composition;

import java.lang.reflect.Method;
import java.util.logging.Logger;

import org.quintet.monk.Attribute;
import org.quintet.backbeat.bandstand.Bandstand;

/** 
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * An <code>Injection</code> is a functor-like object that is capable of injecting a value into a target object.  An <code>injection</code> has a <i>value to inject</i>.  This value is manifested in two ways; a <i>string manifestation</i> and an <i>actual manifestation</i>.  The <i>string manifestation</i> is derived from an XML document.  It is either the literal value that will be injected or it is the name by which the literal value can be obtained from an external source such as JNDI.  For example:<br>
 * <li><code>stringManifestation = "100"</code>, <code>actualValue = 100L</code> -- an <code>Injection</code> that inserts a Long into an object
 * <li><code>stringManifestation = "jdbc/quintet-data-source"</code>, <code>actualValue = {javax.sql.DataSource}</code> -- an <code>injection</code> that uses the <code>stringManifestation</code> to look up a <code>DataSource</code> from the environment (JNDI, for example), and then is capable of inserting that <code>DataSource</code> into an object
 * <br>
 * <code>Injection</code> is an abstract class.  Each concrete subclass is responsible for converting its <code>stringManifestation</code> into its <code>actualValue</code>.  For example, a <code>LongInjection</code> class must convert a string manifestation, such as "100", into a <code>Long</code>.  As mentioned before, some string maifestations are "converted" by treating the string as a name by which to find an object in an external source such as JNDI.  For example, a <code>DataSourceInjection</code> class uses its string manifestation to lookup an actual <code>DataSource</code> from JNDI.
 * <p>
 * @author Peter Thaggard
 * @since Quintet 2 -- January 2009
 * @uml.annotations
 *     derived_abstraction="platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_7uFKsNnQEd2M4cCnHXbxDA"
 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_7uFKsNnQEd2M4cCnHXbxDA"
 */
public abstract class Injection extends Attribute {

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_J7iScNnSEd2M4cCnHXbxDA"
	 */
	private Object actualManifestation;

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the actualManifestation
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_J7iScNnSEd2M4cCnHXbxDA?GETTER"
	 */
	public Object getActualManifestation() {
		// begin-user-code
		return actualManifestation;
		// end-user-code
	}

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param theActualManifestation the actualManifestation to set
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_J7iScNnSEd2M4cCnHXbxDA?SETTER"
	 */
	public void setActualManifestation(Object theActualManifestation) {
		// begin-user-code
		actualManifestation = theActualManifestation;
		// end-user-code
	}

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_btkUMNnTEd2M4cCnHXbxDA"
	 */
	private String setterName;

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the setterName
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_btkUMNnTEd2M4cCnHXbxDA?GETTER"
	 */
	public String getSetterName() {
		// begin-user-code
		return setterName;
		// end-user-code
	}

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param theSetterName the setterName to set
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_btkUMNnTEd2M4cCnHXbxDA?SETTER"
	 */
	public void setSetterName(String theSetterName) {
		// begin-user-code
		setterName = theSetterName;
		// end-user-code
	}

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_04k18NqVEd2M4cCnHXbxDA"
	 */
	private String stringManifestation;

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the stringManifestation
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_04k18NqVEd2M4cCnHXbxDA?GETTER"
	 */
	public String getStringManifestation() {
		// begin-user-code
		return stringManifestation;
		// end-user-code
	}

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param theStringManifestation the stringManifestation to set
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_04k18NqVEd2M4cCnHXbxDA?SETTER"
	 */
	public void setStringManifestation(String theStringManifestation) {
		// begin-user-code
		stringManifestation = theStringManifestation;
		// end-user-code
	}

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_hL_FkNtQEd2Qy-8DGL5OcA"
	 */
	protected static Logger log = Logger
			.getLogger("org.quintet.backbeat.composition");

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the log
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_hL_FkNtQEd2Qy-8DGL5OcA?GETTER"
	 */
	public static Logger getLog() {
		// begin-user-code
		return log;
		// end-user-code
	}

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param target
	 * @param bandstand
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_9NmcsNnQEd2M4cCnHXbxDA"
	 */
	public abstract void injectInto(Object target, Bandstand bandstand);

	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param inSetterName
	 * @param inValue
	 * @generated "sourceid:platform:/resource/Quintet_2/JavaUMLModel1229572370608.emx#_epnOkNnTEd2M4cCnHXbxDA"
	 */
	public Injection(String inSetterName, String inValue) {
		// begin-user-code
		// TODO Auto-generated constructor stub
		this.setterName = inSetterName;
		this.stringManifestation = inValue;
		// end-user-code
	}
}
