package org.quintet.backbeat.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileDescriptor;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.quintet.backbeat.BackBeatException;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * <p>Title:FileInputConduit</p>
 * <p>Description:</p>
 * <code>FileInputConduit</code> is an <code>InputConduit</code> that
 * reads information contained in a file.
 * @version 2.1
 * @since UPS Release 2004.2
 * @see org.quintet.backbeat.io.InputConduit
 */

public class FileInputConduit implements InputConduit {
  /* this class's name used for logging */
  private static final String CLASS = FileInputConduit.CLASS;
  /* logger */
  private static Logger cat = Logger.getLogger("org.quintet.backbeat");

  private BufferedReader input;

  /**
       @roseuid 4113AB820029
   */
  public FileInputConduit() {
  }

  /**
   * @param fileDescriptor - The <code>FileDescriptor</code> from which this
   * <code>FileInputConduit</code> will draw data
   * @roseuid 401807F0001C
   */
  public FileInputConduit(FileDescriptor fileDescriptor) {
    input = new BufferedReader(new FileReader(fileDescriptor));
  }

  /**
   * @param file - The <code>File</code> from which this
   * <code>FileInputConduit</code> will draw data
   * @throws java.io.FileNotFoundException
   * @roseuid 401807F0001A
   */
  public FileInputConduit(File file) throws FileNotFoundException {
    input = new BufferedReader(new FileReader(file));
  }

  /**
   * @param fileName - The name of the file from which this
   * <code>FileInputConduit</code> will draw data
   * @throws java.io.FileNotFoundException
   * @roseuid 401807F00010
   */
  public FileInputConduit(String fileName) throws FileNotFoundException {
    input = new BufferedReader(new FileReader(fileName));
  }

  /**
   * Access method for the input property.

   * @return   the current value of the input property
   */
  public BufferedReader getInput() {
    return input;
  }

  /**
       Sets the value of the input property.

       @param aInput the new value of the input property
   */
  public void setInput(BufferedReader aInput) {
    input = aInput;
  }

  /**
       @return int
       @throws java.io.IOException
       @roseuid 40180EC70299
   */
  public int read() throws IOException {
    return input.read();
  }

  /**
       @param arg0
       @param arg1
       @param arg2
       @return int
       @throws java.io.IOException
       @roseuid 40180EC702A3
   */
  public int read(char[] arg0, int arg1, int arg2) throws IOException {
    return input.read(arg0, arg1, arg2);
  }

  /**
       @return java.lang.String
       @throws java.io.IOException
       @roseuid 40180EC702D5
   */
  public String readLine() throws IOException {
    return input.readLine();
  }

  /**
       @param arg0
       @return long
       @throws java.io.IOException
       @roseuid 40180EC702DF
   */
  public long skip(long arg0) throws IOException {
    return input.skip(arg0);
  }

  /**
       @return boolean
       @throws java.io.IOException
       @roseuid 40180EC702F3
   */
  public boolean ready() throws IOException {
    return input.ready();
  }

  /**
       @return boolean
       @roseuid 40180EC702FD
   */
  public boolean markSupported() {
    return input.markSupported();
  }

  /**
       @param arg0
       @throws java.io.IOException
       @roseuid 40180EC702FE
   */
  public void mark(int arg0) throws IOException {
    input.mark(arg0);
  }

  /**
       @throws java.io.IOException
       @roseuid 40180EC70311
   */
  public void reset() throws IOException {
    input.reset();
  }

  /**
       @throws java.io.IOException
       @roseuid 40180EC7031B
   */
  public void close() throws IOException {
    input.close();
  }

  /**
   * sets up the sepcified file (passed in data) so that this can be read
   * other supplied by <code>FileInputConduit</code>.
   * @param data This could be a String (file path) ,File,FileDescriptor,
   * InputStream
   * @return java.lang.Object
   * @throws org.quintet.backbeat.BackBeatException
   * @roseuid 4113AB820047
   */
  public Object getData(Object data) throws BackBeatException {
    if (null == data) {
      String err = "Precondition 1 failed -- data IS NULL";
      cat.log(Level.WARNING, err);
      return data;
    }
    boolean iShouldClose = false;
        try {
      if (data instanceof String) {
        input = new BufferedReader(new FileReader( (String) data));
        iShouldClose = true;
      } else if (data instanceof File) {
        input = new BufferedReader(new FileReader( (File) data));
        iShouldClose = true;
      } else if (data instanceof FileDescriptor) {
        input = new BufferedReader(new FileReader( (FileDescriptor) data));
        iShouldClose = true;
      } else if (data instanceof InputStream) {
        input = new BufferedReader(new InputStreamReader( (InputStream) data));
        iShouldClose = false;
      } else {
        String err = "expected a String, File, FileDescriptor, or InputStream, " +
            "found a \"" + data.getClass().getName() + "\"";
        cat.log(Level.SEVERE, err);
        throw new BackBeatException(err);
      }
      return input;
    } catch (FileNotFoundException fe) {
      String err = "FileNotFoundException:  " + data.toString();
      cat.log(Level.WARNING, err, fe);
      throw new BackBeatException(err, fe);
    }
  }
}
/**
 int FileInputConduit.available(){
    return 0;
   }


 int FileInputConduit.getFD(){
    return null;
   }


 int FileInputConduit.read(){
    return 0;
   }


 int FileInputConduit.read(byte[]){
    return 0;
   }


 int FileInputConduit.close(){

   }


 int FileInputConduit.skip(long){
    return 0;
   }


 int FileInputConduit.read(byte[],int,int){
    return 0;
   }


 int FileInputConduit.setInput(java.io.FilenameFilter.FileInputStream){
      input = aInput;
   }


 */
