package org.quintet.backbeat.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Writer;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.quintet.backbeat.BackBeatException;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * <p>Title:FileOutputConduit</p>
 * <p>Description:</p>
 * <code>FileOutputConduit</code> is an <code>OutputConduit</code> that
 * transmits information via a file.  The name of the file
 * is configured as an environment entry that has a name "LOCAL_FILE_PATH".
 * This supports multiple incoming data types.
 * <p>Copyright: Copyright (c) 2004</p>
 * @version 2.1
 * @see org.quintet.backbeat.io.OutputConduit
*/

public class FileOutputConduit implements OutputConduit
{
  /* logger */
  private static final Logger cat = Logger.getLogger("org.quintet.backbeat");
  private PrintWriter output;

   /**
   @roseuid 40AA6A880311
    */
   public FileOutputConduit()
   {

   }

   public void closeResources(InitialContext ic){
     if(ic != null){
       try{
         ic.close();
       }catch(Throwable eatIt){}
     }
  }
   /**
   @param arg0
   @param arg1
   @roseuid 401811E102ED
    */
   public FileOutputConduit(OutputStream arg0, boolean arg1)
   {
        output = new PrintWriter(arg0, arg1);
   }

   /**
   @param arg0
   @roseuid 401811E102EB
    */
   public FileOutputConduit(OutputStream arg0)
   {
        output = new PrintWriter(arg0);
   }

   /**
   @param arg0
   @param arg1
   @roseuid 401811E102E3
    */
   public FileOutputConduit(Writer arg0, boolean arg1)
   {
        output = new PrintWriter(arg0, arg1);
   }

   /**
   @param arg0
   @roseuid 401811E102E1
    */
   public FileOutputConduit(Writer arg0)
   {
        output = new PrintWriter(arg0);
   }

   /**
	* Access method for the output property.
	* @return   the current value of the output property
    */
   public PrintWriter getOutput()
   {
        return output;
   }

   /**
	* Sets the value of the output property.
	* @param aOutput the new value of the output property
    */
   public void setOutput(PrintWriter aOutput)
   {
        output = aOutput;
   }

   /**
   @roseuid 401812380191
    */
   public void flush()
   {
        output.flush();
   }

   /**
   @roseuid 40181238019B
    */
   public void close()
   {
        output.close();
   }

   /**
   @return boolean
   @roseuid 4018123801A5
    */
   public boolean checkError()
   {
        return output.checkError();
   }

   /**
   @roseuid 4018123801A6
    */
   protected void setError()
   {
        ;
   }

   /**
   @param arg0
   @roseuid 4018123801AF
    */
   public void write(int arg0)
   {
        output.write(arg0);
   }

   /**
   @param arg0
   @param arg1
   @param arg2
   @roseuid 4018123801C3
    */
   public void write(char[] arg0, int arg1, int arg2)
   {
        output.write(arg0, arg1, arg2);
   }

   /**
   @param arg0
   @roseuid 4018123801EB
    */
   public void write(char[] arg0)
   {
        output.write(arg0);
   }

   /**
   @param arg0
   @param arg1
   @param arg2
   @roseuid 4018123801FF
    */
   public void write(String arg0, int arg1, int arg2)
   {
        output.write(arg0, arg1, arg2);
   }

   /**
   @param arg0
   @roseuid 401812380228
    */
   public void write(String arg0)
   {
        output.write(arg0);
   }

   /**
   @param arg0
   @roseuid 40181238023C
    */
   public void print(boolean arg0)
   {
        output.print(arg0);
   }

   /**
   @param arg0
   @roseuid 401812380250
    */
   public void print(char arg0)
   {
        output.print(arg0);
   }

   /**
   @param arg0
   @roseuid 401812380264
    */
   public void print(int arg0)
   {
        output.print(arg0);
   }

   /**
   @param arg0
   @roseuid 40181238026E
    */
   public void print(long arg0)
   {
        output.print(arg0);
   }

   /**
   @param arg0
   @roseuid 401812380282
    */
   public void print(float arg0)
   {
        output.print(arg0);
   }

   /**
   @param arg0
   @roseuid 401812380296
    */
   public void print(double arg0)
   {
        output.print(arg0);
   }

   /**
   @param arg0
   @roseuid 4018123802AA
    */
   public void print(char[] arg0)
   {
        output.print(arg0);
   }

   /**
   @param arg0
   @roseuid 4018123802BE
    */
   public void print(String arg0)
   {
        output.print(arg0);
   }

   /**
   @param arg0
   @roseuid 4018123802D2
    */
   public void print(Object arg0)
   {
        output.print(arg0);
   }

   /**
   @roseuid 4018123802DC
    */
   public void println()
   {
        output.println();
   }

   /**
   @param arg0
   @roseuid 4018123802E6
    */
   public void println(boolean arg0)
   {
        output.println(arg0);
   }

   /**
   @param arg0
   @roseuid 4018123802FA
    */
   public void println(char arg0)
   {
        output.println(arg0);
   }

   /**
   @param arg0
   @roseuid 40181238030E
    */
   public void println(int arg0)
   {
        output.println(arg0);
   }

   /**
   @param arg0
   @roseuid 401812380322
    */
   public void println(long arg0)
   {
        output.println(arg0);
   }

   /**
   @param arg0
   @roseuid 401812380336
    */
   public void println(float arg0)
   {
        output.println(arg0);
   }

   /**
   @param arg0
   @roseuid 401812380340
    */
   public void println(double arg0)
   {
        output.println(arg0);
   }

   /**
   @param arg0
   @roseuid 401812380354
    */
   public void println(char[] arg0)
   {
        output.println(arg0);
   }

   /**
   @param arg0
   @roseuid 401812380368
    */
   public void println(String arg0)
   {
        output.println(arg0);
   }

   /**
   @param arg0
   @roseuid 40181238037C
    */
   public void println(Object arg0)
   {
        output.println(arg0);
   }

   /**
   @param data
   @return java.lang.Object
   @throws org.quintet.backbeat.BackBeatException
   @roseuid 411A4C3C0138
    */
   public Object writeData(Object data) throws BackBeatException
   {
       BufferedReader br = null;
       BufferedWriter bw = null;
       InitialContext ic = null;
       try
       {
           ic = new InitialContext();
           String localFilePath =
                (String)ic.lookup("java:comp/env/LOCAL_FILE_PATH");
           bw = new BufferedWriter(new FileWriter(new File(localFilePath)));

           cat.log(Level.FINEST,"Data class: " + data.getClass().getName());
           if(data instanceof String)
           {
               bw.write((String)data);
           }
           else if(data instanceof FileInputStream)
           {
               br = new BufferedReader(new InputStreamReader((FileInputStream)data));
           }
           else if(data instanceof InputStreamReader)
           {
               br = new BufferedReader((InputStreamReader)data);
           }
           else if(data instanceof File)
           {
               br = new BufferedReader(new FileReader((File)data));
           }
           else if(data instanceof FileReader)
           {
               br = new BufferedReader((FileReader)data);
           }
           else if(data instanceof BufferedReader)
           {
               br =(BufferedReader)data;
           }

           if(br != null)
           {
               cat.log(Level.FINEST,"Writing data");
               String line = br.readLine();
               if(line != null)
               {
                   bw.write(line);
                   line = br.readLine();
                   while(line != null)
                   {
                       bw.newLine();
                       bw.write(line);
                       line = br.readLine();
                   }
               }
           }

       }
       catch(NamingException ne)
       {
         cat.log(Level.SEVERE,ne.getMessage(),ne);
         closeResources(ic);
           throw new BackBeatException("Error in looking up environment entries: " +
               ne.getMessage());
       }
       catch(IOException ie)
       {
		   cat.log(Level.SEVERE,ie.getMessage(),ie);
           closeResources(ic);
           throw new BackBeatException("Error in looking up environment entries: " +
               ie.getMessage());
       }
       finally
       {
               if(br != null)
               {
                 try{
                   br.close();
                 }catch(IOException eatIt){}
               }
               if(bw != null)
               {
                 try{
                   bw.close();
                 }catch(IOException eatIt){}
               }
       }
       closeResources(ic);
       return null;
   }
}
