package org.quintet.backbeat.io;

import org.quintet.chambers.EntityRetriever;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.File;
import org.quintet.chambers.RetrieverException;
import java.util.logging.Logger;
import java.util.logging.Level;


/**
 * <p>Title: FileRetriever</p>
 * <p>Description:</p>
 * <code>FileRetriever</code> accepts a <code>fileName</code>, opens the file
 * designated by the name, and returns the file as a
 * <code>FileInputStream</code>.
 * <p>Copyright: Copyright (c) 2004</p>
 * @version 2.1
 */
public class FileRetriever
    implements EntityRetriever {


	/* logger */
	private static Logger log = Logger.getLogger("org.quintet.backbeat");

  /**
      @roseuid 41223D51028C
   */
  public FileRetriever() {

  }

  /**
      @param pk
      @return java.io.FileInputStream;
      @throws org.quintet.chambers.RetrieverException
      @roseuid 41223D510296
   */
  public Object retrieve(Object pk) throws RetrieverException {
    try {
      if (pk == null) {
    	  String warn = "Precondition 1 failed -- PK IS NULL";
    	  log.log(Level.WARNING,warn);
    	  return pk;
      }
      
      if (pk instanceof File) {
        FileInputStream answer = new FileInputStream( (File) pk);
        return answer;
      }
      else if (pk instanceof java.lang.String) {
        FileInputStream answer = new FileInputStream( (String) pk);
        return answer;
      }
      else if (pk instanceof java.io.InputStream) {
        return (java.io.InputStream) pk;
      }
      else {
    	  String err = "Expected a String, File, or InputStream, "+
    	  "received a " + pk.getClass().getName();
    	  log.log(Level.SEVERE,err);
    	  throw new RetrieverException(err);
      }
    }
    catch (IOException ioe) {
    	String err = "Unable to open a file named \"" +
    	pk.toString() + "\"";
    	log.log(Level.SEVERE,err,ioe);
    	throw new RetrieverException(err,ioe);
    }
  }
}
