package org.quintet.backbeat.io;
import java.net.URLConnection;
import java.net.URL;
import org.quintet.backbeat.BackBeatException;
import java.net.MalformedURLException;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.io.InputStream;
import java.io.*;
import java.util.Map;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.auth.DigestScheme;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import java.net.URLEncoder;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.auth.AuthPolicy;
import java.util.List;
import java.util.ArrayList;

/**
 * <code>ServletConduit</code> provides a standardized means of invoking a
 * servlet's <code>GET</code> method and retrieving the <code>byte[]</code>
 * response from the servlet.
 *
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * <p>Company: </p>
 *
 * @author Peter Thaggard
 * @version 1.0
 */
public class ServletConduit implements InputConduit{

    private static final Logger log = Logger.getLogger("org.quintet.backbeat");
    private String baseURL = null;
    private HttpClient client = null;
    private String userName = null;
    private String md5Credentials = null;

    public ServletConduit(String inBaseURL, String inUserName, String inMD5Credentials){
        baseURL = inBaseURL;
        userName = inUserName;
        md5Credentials = inMD5Credentials;
        try {
            client = new HttpClient();
        } catch (Throwable ex) {
            String err = ex.getClass().getName() +
                         " when creating an HttpClient()";
            log.log(Level.SEVERE,err,ex);
            throw new IllegalStateException(err,ex);
        }
    }

    public static final String STREAM_MAP_ENTRY = "___servletConduitDataStream___";

    public Object getData(Object data) throws BackBeatException {
        GetMethod conn = null;
        StringBuffer fullURL = new StringBuffer(baseURL).append("?");
        try { //wrap the whole method in a try/catch/finally so we can clean up
            //data must be a Map<String,Object>.  Each entry is a name/value
            //pair that becomes a param to be passed to the servlet, unless
            //the "name" part is ___servletConduitDataStream___, in which case
            //is is an InputStream to be connected to the Servlet

            InputStream streamToServlet = null;

            if (data == null) {
                String warn = "data object is null, therefore no parameters " +
                              "will be passed to the servlet";
            } else {
                if (!(data instanceof Map)) {
                    String err = "expected a java.util.Map, found a " +
                                 data.getClass().getName();
                    log.log(Level.WARNING, err);
                    return null;
                }
                Map paramMap = (Map) data;
                int numParams = paramMap.size();
                int numProcessed = 0;
                for (Object nameObj : paramMap.keySet()) {
                    if (!(nameObj instanceof String)) {
                        String err = "expected a name object of type String, " +
                                     "found a \"" + nameObj.getClass().getName() +
                                     "\"";
                        log.log(Level.SEVERE, err);
                        throw new BackBeatException(err);
                    }
                    String name = (String) nameObj;
                    Object rawValue = paramMap.get(name);
                    if (name.equals(STREAM_MAP_ENTRY)) {

                        numParams--;
                    } else {
                        if (!(rawValue instanceof String)) {
                            String err = "parameter \"" + name +
                                         "\" is associated " +
                                         "with a value of type \"" +
                                         rawValue.getClass().getName() +
                                         ", but only " +
                                         "values of type \"String\" are allowed.";
                            log.log(Level.SEVERE, err);
                            throw new BackBeatException(err);
                        }
                        numProcessed++;
                        String oneChunk = name + "=" + URLEncoder.encode((String)rawValue,"UTF-8");
                        fullURL.append(oneChunk);
                        if (numProcessed < numParams)
                            fullURL.append("&");
                    }
                }
            }
            fullURL.append("&loginid="+URLEncoder.encode(userName,"UTF-8"));
            //okey dokey, full URL, including parameters, is set up.
            String encodedURL = fullURL.toString();
            log.log(Level.INFO,"fullURL is \"" + fullURL + "\"");

            /*UsernamePasswordCredentials credentials = new UsernamePasswordCredentials(
                    userName,md5Credentials
            );
            List digestPreferred = new ArrayList();
            digestPreferred.add(AuthPolicy.DIGEST);
            digestPreferred.add(AuthPolicy.BASIC);
            client.getParams().setParameter(AuthPolicy.AUTH_SCHEME_PRIORITY, digestPreferred);
            client.getState().setCredentials(
                    AuthScope.ANY,credentials);
*/
            conn = new GetMethod(encodedURL);
            conn.setRequestHeader("Content-type", "text/javascript");
            String creds = "todd@6thsensesoftware.com:todd123";
            conn.setRequestHeader("Authorization", "Basic " + new String(Base64.encodeBase64(creds.getBytes())));

            int status = client.executeMethod(conn);

            if(status < 200 || status > 399){
                String err = "got HTTP status \""+status+"\", which indicates failure";
                log.log(Level.SEVERE,err);
                return null;
            }else{
                byte[] answer = conn.getResponseBody();
                return answer;
            }
        }catch(Throwable t){
            Throwable cause = t.getCause() == null ? t:t.getCause();
            String err = cause.getClass().getName() + "when conversing with "+
                         "HTTP resource at address \""+fullURL.toString()+"\"";
            log.log(Level.SEVERE,err,cause);
            throw new BackBeatException(err,cause);
        }finally{
            try{
            conn.releaseConnection();
            }catch(Throwable eatIt){}
        }
    }
}
