package org.quintet.backbeat.io;

import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.logging.Logger;
import java.util.logging.Level;
import org.quintet.QuintetException;
/**
 * <p>Title:</p>
 * <p>Description:</p>
 * <code>StaticFileInputConduit</code> is a <code>FileInputConduit</code>
 * that reads a configuration-driven file; that is, it always opens and
 * reads from the exact same file.  <code>StaticFileInputConduit</code>
 * contrasts with <code>DynamicFileInputConduit</code>, which uses
 * configuration parameters to dynamically read from a file every
 * time its <code>getData</code> method is invoked.
 * <p>Copyright: Copyright (c) 2004</p>
 * @version 2.1
 * @see org.quintet.backbeat.io.InputConduit
 * @see org.quintet.backbeat.io.FileInputConduit
 */

public class StaticFileInputConduit extends FileInputConduit {

  /* logger */
  private static final Logger log = Logger.getLogger("org.quintet.backbeat");
  /**
      @roseuid 40AA510F0378
   */
  public StaticFileInputConduit() {
    InitialContext ic = null;
    BufferedReader br = null;
    FileReader fr = null;
    try {
      ic = new InitialContext();
    }catch(NamingException ne){
      String err = "Unable to obtain a JNDI InitialContext -- either the "+
          "App Server is down or (more likely) you are trying to use this "+
          "class outside of an App Server.  This class ONLY WORKS WITHIN a "+
          "J2EE App Server.";
      log.log(Level.SEVERE,err,ne);
      throw new IllegalStateException(err,ne);
    }
    String inputFileName = null;
    final String INPUT_FILE_REF = "java:comp/env/input-file-name";
    try{
      inputFileName = (String) ic.lookup(INPUT_FILE_REF);
      fr = new FileReader(inputFileName);
      br = new BufferedReader(fr);
      setInput(br);
    } catch (NamingException ne) {
      String err = "Unable to find \"" + INPUT_FILE_REF + "\" "+
          "within JNDI.  Please check the deployment descriptor.";
      log.log(Level.SEVERE,err,ne);
      throw new IllegalStateException(err,ne);
    } catch (IOException ioe) {
      String err = "IOException when opening/reading \"" +
          inputFileName + "\"";
      log.log(Level.SEVERE,err,ioe);
      throw new IllegalStateException(err,ioe);
    }
  }
}
