package org.quintet.backbeat.io;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * <p>Title:StaticFileOutputConduit</p>
 * <p>Description:</p>
 * <code>StaticFileOutputConduit</code> is a <code>FileOutputConduit</code>
 * that manipulates a configuration-driven file; that is, it always opens and
 * writes to the exact same file.  <code>StaticFileOutputConduit</code>
 * contrasts with <code>DynamicFileOutputConduit</code>, which uses
 * configuration parameters to dynamically create a new output file every
 * time its <code>write...</code> methods are invoked.
 * <p>Copyright: Copyright (c) 2004</p>
 * @version 2.1
 * @see org.quintet.backbeat.io.OutputConduit
 * @see org.quintet.backbeat.io.FileOutputConduit
 */
public class StaticFileOutputConduit extends FileOutputConduit {

  /* logger */
  private static final Logger log = Logger.getLogger("org.quintet.backbeat");

  /**
      @roseuid 40AA1F9A00B3
   */
  public StaticFileOutputConduit() {
    log.log(Level.FINEST, "ctor");
    final String OUTPUT_FILE_REF = "java:comp/env/output-file-path";
    InitialContext ic = null;
    FileOutputStream fos = null;
    PrintWriter pw = null;
    try {
      ic = new InitialContext();
    }catch(NamingException ne){
      String err = "Unable to obtain a JNDI InitialContext -- either the "+
          "App Server is down or (more likely) you are trying to use this "+
          "class outside of an App Server.  This class ONLY WORKS WITHIN a "+
          "J2EE App Server.";
      log.log(Level.SEVERE,err,ne);
      throw new IllegalStateException(err,ne);
    }

    String outputFileName = null;
    try{
      outputFileName = (String) ic.lookup(OUTPUT_FILE_REF);
      fos = new FileOutputStream(outputFileName);
      pw = new PrintWriter(fos);
      this.setOutput(new PrintWriter(fos));
    } catch (NamingException ne) {
      String err = "Unable to find \"" + OUTPUT_FILE_REF + "\" " +
          "within JNDI.  Please check your deployment descriptor.";
      log.log(Level.SEVERE,err,ne);
      throw new IllegalStateException(err,ne);
    } catch (IOException ioe) {
      String err = "IOException when opening file \"" +
          outputFileName + "\".";
      throw new IllegalStateException(err,ioe);
    }finally{
      if(ic != null){
        try{
          ic.close();
        }catch(Throwable eatIt){}
      }
      if(pw != null){
        try{
          pw.close();
        }catch(Throwable eatIt){}
      }
      if(fos != null){
        try{
          fos.close();
        }catch(Throwable eatIt){}
      }
    }
  }
}
