package org.quintet.backbeat.supplier;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.quintet.chambers.EntityRetriever;
import org.quintet.backbeat.dataportal.DataPortal;
import org.quintet.backbeat.io.ForwardConduit;
import org.quintet.backbeat.transformer.Transformer;
import org.quintet.backbeat.validation.Validator;
import java.sql.Timestamp;

import java.util.logging.Logger;
import java.util.logging.Level;

public class SessionSupplierBean
    extends Supplier
    implements SessionBean {

	static final long servierVeruidUID = 200501001;
  private String processDescription = "<no description>";
  private static final Logger log = Logger.getLogger("org.quintet.backbeat");

  private static final String DATA_PORTAL_CLASS_NAME_ENV_ENTRY =
      "java:comp/env/data-portal-class-name";
  private static final String FORWARD_CONDUIT_CLASS_NAME_ENV_ENTRY =
      "java:comp/env/forward-conduit-class-name";
  private static final String RETRIEVER_CLASS_NAME_ENV_ENTRY =
      "java:comp/env/request-retriever-class-name";
  private static final String REQUEST_CONDUIT_CLASS_NAME_ENV_ENTRY =
      "java:comp/env/request-conduit-class-name";
  private static final String RESPONSE_CONDUIT_CLASS_NAME_ENV_ENTRY =
      "java:comp/env/response-conduit-class-name";
  private static final String REQUEST_TRANSFORMER_CLASS_NAME_ENV_ENTRY =
      "java:comp/env/outbound-transformer-class-name";
  private static final String RESPONSE_TRANSFORMER_CLASS_NAME_ENV_ENTRY =
      "java:comp/env/inbound-transformer-class-name";
  private static final String RESPONSE_VALIDATOR_CLASS_NAME_ENV_ENTRY =
      "java:comp/env/response-validator-class-name";

  /**
      The container-supplied context information
   */
  private SessionContext context;

  /**
      @roseuid 411B617A0158
   */
  public SessionSupplierBean() {

  }

  /**
      @param inRequestID - The ID of the request that goaded this
      <code>Supplier</code> into action
      @return java.lang.Object
      @throws org.quintet.backbeat.supplier.SupplierException
      @roseuid 411A3C6B034F
   */
  public Object acquireInformation(String inRequestID) throws SupplierException {
    String thisMethod = ".acquireInformation";
    Timestamp now = new Timestamp(System.currentTimeMillis());
    log.info(processDescription + ":" + now + ":starting");
    Object answer;
    try{
      answer = super.acquireInformation(inRequestID);
    }catch(SupplierException se){
      now = new Timestamp(System.currentTimeMillis());
      String err = processDescription + ":" + now + ":caught SupplierException";
      log.log(Level.SEVERE,err,se);
      throw se;
    }
    now = new Timestamp(System.currentTimeMillis());
    log.info(processDescription + ":" + now + ":done");
    return answer;
  }

  /**
      @param requestIdentifier
      @return java.lang.Object
      @roseuid 411B617A016C
   */
  /*protected Object retrieveRequest(Object requestIdentifier) {
    return requestIdentifier;
  }*/

  /**
      @param internalForm
      @return java.lang.Object
      @roseuid 411B617A0180
   */
  //protected Object forwardResponse(Object internalForm) {
  //  return internalForm;
  //}

  /**
      @throws javax.ejb.EJBException
      @throws java.rmi.RemoteException
      @roseuid 411B617A0194
   */
  public void ejbActivate() throws EJBException, RemoteException {

  }

  /**
      @throws javax.ejb.EJBException
      @throws java.rmi.RemoteException
      @roseuid 411B617A019E
   */
  public void ejbPassivate() throws EJBException, RemoteException {

  }

  /**
      @throws javax.ejb.EJBException
      @throws java.rmi.RemoteException
      @roseuid 411B617A01A8
   */
  public void ejbRemove() throws EJBException, RemoteException {

  }


  public void closeResources(InitialContext ic){
    if(ic != null){
      try{
        ic.close();
      }catch(Throwable eatIt){}
    }
  }
  /**
      @param sessioncontext
      @throws javax.ejb.EJBException
      @roseuid 411B617A01B2
   */
  public void setSessionContext(SessionContext sessioncontext) throws
      EJBException {
    context = sessioncontext;
    //configure this supplier's DataPortal, Transformers, and Validator
    InitialContext ic = null;

    try {
      ic = new InitialContext();
    }
    catch (NamingException ex) {
      throw new EJBException(
          "Unable to get an initialContext:" +
          ex);
    }

    //-----------------  configure DataPortal ----------------------
    String dataPortalClassName = null;
    try {
      dataPortalClassName = (String) ic.lookup(
          DATA_PORTAL_CLASS_NAME_ENV_ENTRY);
    }
    catch (NamingException ex1) {
      throw new EJBException(
          "SessionSupplierBean:setSessionContext:Unable to lookup data portal named " +
          DATA_PORTAL_CLASS_NAME_ENV_ENTRY);
    }
    log.log(Level.FINEST,"dataPortalClassName:" +
                       dataPortalClassName);
    if (dataPortalClassName != null && !dataPortalClassName.equals("") &&
        !dataPortalClassName.equals("null")) {
      Class dataPortalClass = null;
      try {
        dataPortalClass = Class.forName(dataPortalClassName);
      }
      catch (ClassNotFoundException ex2) {
        closeResources(ic);
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:Unable to find a class named " +
            dataPortalClassName);
      }
      try {
        DataPortal myDataPortal = (DataPortal) dataPortalClass.newInstance();
        setDataPortal(myDataPortal);
      }
      catch (IllegalAccessException ex3) {
        String err = "IllegalAccessException when instantiating DataPortal";
        log.log(Level.SEVERE,err,ex3);
        closeResources(ic);
        throw new EJBException(err,ex3);
      }
      catch (InstantiationException ex3) {
        String err = "InstantiationException when instantiating DataPortal";
        log.log(Level.SEVERE,err,ex3);
        throw new EJBException(err,ex3);

      }catch(Exception eh){
        String err = "Exception when instantiating DataPortal";
        log.log(Level.SEVERE,err,eh);
        throw new EJBException(err,eh);
      }
    }
    else {
      log.log(Level.FINEST,
          "SessionSupplierBean:WARNING:setting data portal to null");
      setDataPortal(null);
    }

    //------------------------ configure request retriever --------------------------------
    String retrieverClassName = null;
    try {
      retrieverClassName = (String) ic.lookup(RETRIEVER_CLASS_NAME_ENV_ENTRY);
    }
    catch (NamingException ne) {
      throw new EJBException(
          "SessionSupplierBean:setSessionContext:unable to lookup retrieve named " +
          RETRIEVER_CLASS_NAME_ENV_ENTRY + ":" + ne);
    }
    log.log(Level.FINEST,
        "SessionSupplierBean:setSessionContext:INFO:retrieverClassName:" +
        retrieverClassName);
    if (retrieverClassName != null && !retrieverClassName.equals("") &&
        !retrieverClassName.equalsIgnoreCase("null")) {
      Class retrieverClass = null;
      try {
        retrieverClass = Class.forName(retrieverClassName);
      }
      catch (ClassNotFoundException ex) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:unable to find class named " +
            retrieverClassName + ":" + ex);
      }
      try {
        EntityRetriever _retriever = (EntityRetriever) retrieverClass.
            newInstance();
        setRetriever(_retriever);
      }
      catch (IllegalAccessException ex) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:IllegalAccessException on " +
            retrieverClassName + ":" + ex);
      }
      catch (InstantiationException ex) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:InstantiationException on " +
            retrieverClassName + ":" + ex);
      }
    }
    else {
      log.log(Level.FINEST,
          "SessionSupplierBean:setSessionContext:WARNING:setting retriever to NULL");
      setRetriever(null);
    }
    //---------------------- configure request transformer -------------------------------
    String requestTransformerClassName = null;
    try {
      requestTransformerClassName = (String) ic.lookup(
          REQUEST_TRANSFORMER_CLASS_NAME_ENV_ENTRY);
    }
    catch (NamingException ex4) {
      throw new EJBException("SessionSupplierBean:setSessionContext:unable to lookup request transformer named " +
                             REQUEST_TRANSFORMER_CLASS_NAME_ENV_ENTRY);
    }
    if (requestTransformerClassName != null &&
        !requestTransformerClassName.equals("") &&
        !requestTransformerClassName.equals("null")) {
      Class requestTransformerClass = null;
      try {
        requestTransformerClass = Class.forName(requestTransformerClassName);
      }
      catch (ClassNotFoundException ex5) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:Unable to find a class named " +
            requestTransformerClassName);
      }
      try {
        Transformer requestTransformer = (Transformer) requestTransformerClass.
            newInstance();
        this.setOutboundTransformer(requestTransformer);
      }
      catch (IllegalAccessException ex6) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:IllegalAccessException on " +
            requestTransformerClassName + ":" + ex6);
      }
      catch (InstantiationException ex6) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:InstantiationException on " +
            requestTransformerClassName + ":" + ex6);
      }
    }
    else {
      setOutboundTransformer(null);
    }
    //------------------------- configure response transformer --------------------------------------------
    String responseTransformerClassName = null;
    try {
      responseTransformerClassName = (String) ic.lookup(
          RESPONSE_TRANSFORMER_CLASS_NAME_ENV_ENTRY);
    }
    catch (NamingException ex7) {
      throw new EJBException("SessionSupplierBean:setSessionContext:unable to lookup response transformer named " +
                             RESPONSE_TRANSFORMER_CLASS_NAME_ENV_ENTRY);
    }
    if (responseTransformerClassName != null &&
        !responseTransformerClassName.equals("") &&
        !responseTransformerClassName.equals("null")) {
      Class responseTransformerClass = null;
      try {
        responseTransformerClass = Class.forName(responseTransformerClassName);
      }
      catch (ClassNotFoundException ex8) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:unable to find class named " +
            responseTransformerClassName);
      }
      try {
        Transformer responseTransformer = (Transformer)
            responseTransformerClass.newInstance();
        this.setInboundTransformer(responseTransformer);
      }
      catch (IllegalAccessException ex9) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:IllegalAccessException from " +
            responseTransformerClassName + ":" + ex9);
      }
      catch (InstantiationException ex9) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:InstantiationException from " +
            responseTransformerClassName + ":" + ex9);
      }
    }
    else {
      setInboundTransformer(null);
    }
    //-----------------  configure ForwardConduit ----------------------
    String forwardConduitClassName = null;
    try {
        forwardConduitClassName = (String) ic.lookup(
          FORWARD_CONDUIT_CLASS_NAME_ENV_ENTRY);
    }
    catch (NamingException ex1) {
      //@Todo need to throw following exception after modifying deployment
      //descriptors of existing session suppliers
      //throw new EJBException(
      //    "SessionSupplierBean:setSessionContext:Unable to lookup forward conduit class " +
      //    FORWARD_CONDUIT_CLASS_NAME_ENV_ENTRY);
      log.log(Level.FINEST,"SessionSupplierBean:" + "Forward Conduit not configured");
    }
    log.log(Level.FINEST,"SessionSupplierBean:forwardConduitClassName:" +
        forwardConduitClassName);
    if (forwardConduitClassName != null && !forwardConduitClassName.equals("") &&
        !forwardConduitClassName.equals("null")) {
      Class forwardConduitClass = null;
      try {
          forwardConduitClass = Class.forName(forwardConduitClassName);
      }
      catch (ClassNotFoundException ex2) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:Unable to find a class named " +
            forwardConduitClassName);
      }
      try {
          ForwardConduit myForwardConduit = (ForwardConduit) forwardConduitClass.newInstance();
          setForwardConduit(myForwardConduit);
      }
      catch (IllegalAccessException ex3) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:IllegalAccessException on " +
            forwardConduitClassName + ":" + ex3);
      }
      catch (InstantiationException ex3) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:InstantiationException on " +
            forwardConduitClassName + ":" + ex3);
      }
    }
    else {
      log.log(Level.FINEST,
          "SessionSupplierBean:WARNING:setting forward conduit to null");
      setForwardConduit(null);
    }


    //------------------------ configure validator --------------------------
    String validatorClassName = null;
    try {
      validatorClassName = (String) ic.lookup(SessionSupplierBean.
                                              RESPONSE_VALIDATOR_CLASS_NAME_ENV_ENTRY);
    }
    catch (NamingException ex10) {
      throw new EJBException(
          "SessionSupplierBean:setSessionContext:unable to lookup " +
          RESPONSE_VALIDATOR_CLASS_NAME_ENV_ENTRY);
    }
    if (validatorClassName != null && !validatorClassName.equals("") &&
        !validatorClassName.equals("null")) {
      Class validatorClass = null;
      try {
        validatorClass = Class.forName(validatorClassName);
      }
      catch (ClassNotFoundException ex11) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:unable to find class " +
            validatorClassName);
      }
      try {
        Validator validator = (Validator) validatorClass.newInstance();
        log.log(Level.FINEST,"Instantiated a Validator of type " + validator.getClass().getName());
      }
      catch (IllegalAccessException ex12) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:IllegalAccessException from " +
            validatorClassName + ":" + ex12);
      }
      catch (InstantiationException ex12) {
        throw new EJBException(
            "SessionSupplierBean:setSessionContext:InstantiationException from " +
            responseTransformerClassName + ":" + ex12);
      }
    }
    else {
    }
    try{
      processDescription = (String)ic.lookup("java:comp/env/process-description");
    }catch(NamingException ne){
      processDescription = "<unknown process>";
    }
  }

  /**
      @throws javax.ejb.CreateException
      @roseuid 411BC4330213
   */
  public void ejbCreate() throws CreateException {

  }
}
