package org.quintet.backbeat.transformer;

import org.quintet.ellington.Arranger;
import org.quintet.ellington.JNDIArranger;
import org.quintet.chambers.EntityBeanRetriever;
import java.util.Map;
import java.util.logging.Logger;
import java.util.logging.Level;
import org.quintet.chambers.*;
import org.quintet.*;


/**
 * <code>IDToEntityTransformer</code> accepts an entity bean instance's
 * <i>internal ID</i> and returns the corresponding entity instance.  For example,
 * one might pass in a <code>Long</code> <code>Principal</code> ID and be passed
 * back the corresponding <code>Principal</code>
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: 6th Sense Analytics</p>
 *
 * @author Peter Thaggard
 * @version 1.0
 */
public class IDToEntityTransformer implements Transformer {

  private static final Logger log = Logger.getLogger("org.quintet.backbeat");

  private EntityBeanRetriever retriever = null;

  private Arranger arranger = new JNDIArranger();

  private static final String[] propertyNames = {
      "entity-retriever-pk-class-name"
  };

  private String pkClassName = null;

  public IDToEntityTransformer() {
    try {
      retriever = new EntityBeanRetriever();
    } catch (RetrieverException ex) {
      String err = "RetrieverException when trying to create a new "+
          "EntityBeanRetriever";
      log.log(Level.SEVERE,err,ex);
      throw new IllegalStateException(err,ex);
    }
    Map<String, Object> properties = null;
    try {
      properties = arranger.arrange(propertyNames);
    } catch (QuintetException ex1) {
      String err = "Unable to read properties from JNDIArranger";
      log.log(Level.SEVERE,err);
      throw new IllegalStateException(err,ex1);
    }

    pkClassName = (String)properties.get(propertyNames[0]);
    if(null == pkClassName){
      String err = "Unable to find a value for property \"" +
          propertyNames[0] + "\"";
      log.log(Level.SEVERE,err);
      throw new IllegalStateException(err);
    }
  }

  /**
   * Transform <code>inRawData</code> to a new manifestation and return the new
   * manifestation to the client.
   *
   * @param inRawData - The raw data to be transformed to a new manifestation
   * @return java.lang.Object
   * @throws TransformerException
   * @todo Implement this org.quintet.backbeat.transformer.Transformer method
   */
  public Object transform(Object inRawData) throws TransformerException {
    if(null == inRawData){
      String warn = "Unable to transform raw objects, returning null.";
      log.log(Level.WARNING,warn);
      return null;
    }

    if(inRawData.getClass().getName().equals(pkClassName) == false){
      String err = "Expected a \"" + pkClassName + "\", found a \"" +
          inRawData.getClass().getName() + "\"";
      log.log(Level.SEVERE,err);
      throw new TransformerException(err);
    }

    try {
      return retriever.retrieve(inRawData);
    } catch (RetrieverException ex) {
      String err = "RetrieverException when trying to retrieve object with PK "+
          "\"" + inRawData.toString() + "\"";
      log.log(Level.SEVERE,err);
      throw new TransformerException(err,ex);
    }
  }
}
