package org.quintet.backbeat.transformer;

import java.util.Collection;
import java.util.Map;
import java.util.ArrayList;
import java.beans.BeanInfo;
import java.util.HashMap;
import java.beans.PropertyDescriptor;
import java.util.logging.Logger;import java.util.logging.Level;
import java.util.Iterator;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

import org.quintet.backbeat.io.Email;

/**
 * <p> </p>
 *
 * <p> </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * @author Peter Thaggard
 * @version 1.0
 */
public class JavaBeanCollectionToEmail implements Transformer {
  public JavaBeanCollectionToEmail() {
  }

  private static Logger log = Logger.getLogger("org.quintet.backbeat");

  /**
      <code>JavaBeanCollectionToMapCollection</code> accepts a
   <code>Collection</code> of JavaBeans-compliant objects.  It converts the collection
   * into a number of text lines, which it then deposits into a
   * <code>org.quintet.backbeat.io.Email</code> instance.
      This method returns an <code>Email</code> instnace
   @param inRawData - The <code>Collection</code> of JavaBeans-compliant objects
      that will be transformed into an <code>Email</code>
      @return Object
      @throws TransformerException
      @roseuid 420277ED01AB
   */
  public Object transform(Object inRawData) throws TransformerException {
    if (inRawData == null) {
      return null;
    }
    Collection input = null;
    if ( (inRawData instanceof Collection) == false) {
      String warn = "Expected a java.util.Collection, got a " +
          inRawData.getClass().getName() + ".  Wrapping it in a Collection on your behalf.";
      log.log(Level.WARNING,warn);
      input = new ArrayList(1);
      input.add(inRawData);
    } else {
      input = (Collection) inRawData;
    }
    Email answer = new Email();
    StringBuffer output = new StringBuffer();
    for (Iterator inputIter = input.iterator(); inputIter.hasNext(); ) {
      Object currBean = inputIter.next();
      if (currBean == null) {
        continue;
      }
      try {
        BeanInfo beanInfo = Introspector.getBeanInfo(currBean.getClass());
        if (beanInfo == null) {
          continue;
        }
        PropertyDescriptor[] properties = beanInfo.getPropertyDescriptors();
        if (properties == null || properties.length == 0) {
          continue;
        }
        for (int propIdx = 0; propIdx < properties.length; propIdx++) {
          PropertyDescriptor currProperty = properties[propIdx];
          String propertyName = currProperty.getName();
          if (propertyName.equals("class")) {
            continue; //not a good idea to try to flatten a java.lang.Class
          }
          Method getter = currProperty.getReadMethod();
          if (getter == null) {
            output.append(propertyName + ":  [null]\n");
          } else {
            Class declaredIn = getter.getDeclaringClass();
            if (declaredIn.equals(currBean.getClass()) == false) {
              String info = getter.getName() + " not declared in " + currBean.getClass().getName() + ", skipping....";
              log.info(info);
              continue;
            }
            try {
              Object propertyValue = getter.invoke(currBean, (Object[])null);
              output.append(propertyName + ":  " + propertyValue.toString() + "\n");

            } catch (InvocationTargetException ex1) {
              Throwable orig = ex1.getCause();
              String err = "Method " + getter.getName() + " of class " +
                  currBean.getClass().getName() + " threw an exception:  " +
                  orig;
              log.log(Level.SEVERE,err, orig);
              throw new TransformerException(err, ex1);
            } catch (IllegalArgumentException ex1) {
              String err = "IllegalArgumentException when invoking " +
                  currBean.getClass().getName() + "." + getter.getName();
              log.log(Level.SEVERE,err, ex1);
              throw new TransformerException(err, ex1);
            } catch (IllegalAccessException ex1) {
              String err = "IllegalArumentException when invoking " +
                  currBean.getClass().getName() + "." + getter.getName();
              log.log(Level.SEVERE,err, ex1);
              throw new TransformerException(err, ex1);

            }
          }
        }
      } catch (IntrospectionException ex) {
        String err = "Unable to introspect object of class " + currBean.getClass().getName();
        log.log(Level.SEVERE,err, ex);
        throw new TransformerException(err, ex);
      }
    }
    answer.setBody(output);
    return answer;
  }
}
