
package org.quintet.chambers;

import org.quintet.QuintetException;

/**
 * <p>Title: EntityException</p>
 * <p>Description: <code>EntityException</code> is the base of all exceptions
 * thrown by
 * members of the entity package. Like all good exceptions it provides a means to
 * wrap
 * an "original" exception.</p>
 * @author PThaggard
 * @version 1.0
 * @see QuintetException
 */
public class EntityException
    extends QuintetException {
	static final long serialVersionUID = 200501001;

  /**
   * Construct an <code>EntityException</code> the the message contained in
   * <code>inMsg</code>.
   * Arrange for the new <code>EntityException</code> to contain a reference to an
   * "original"
   * exception.
   * @param inMsg String  The exception's diagnostic message
   * @param orig Exception The exception being wrapped by this
   * <code>EntityException</code>
   * @roseuid 41B9D1E3038F
   */
  public EntityException(String inMsg, Exception orig) {

    super(inMsg, orig); //nothing to do here except call the base class's constructor
  }

  /**
   * Construct an <code>EntityException</code> with the message contained in
   * <code>inMsg</code>
   * @param inMsg String -- The newly-constructed exception's diagnostic message
   * @roseuid 41B9D1E30387
   */
  public EntityException(String inMsg) {
    super(inMsg); //nothing to do here except call the base class's constructor
  }

  /**
   * Construct an <code>EntityException</code> with no message and no original
   * exception*
   * @roseuid 41B9D1E30386
   */
  public EntityException() {
    super(); //nothing to do here except call the base class's constructor
  }
}
