package org.quintet.chambers;

import javax.ejb.EJBObject;
import java.rmi.RemoteException;
import java.io.Serializable;
import org.quintet.chambers.EntityException;

/**
<code>EntityFactory</code> is a session bean front-end to
<code>Monk</code>.  <code>EntityFactory</code> lives within a WebLogic
JVM, therefore it is capable of constructing local entity beans, which
<code>Monk</code> is unable to do on its own.  This is merely the component
*interface to <code>EntityFactoryBean</code>, so most of the interesting documentation will
* be found in that class's JavaDoc.
* @author PThaggard
* @see org.quintet.chambers.EntityFactoryBean
* @see org.quintet.monk.Monk
 */
public interface EntityFactory extends EJBObject
{

   /**
   <code>convertToEntity</code> accepts a String containing XML and converts its
   contents to an entity bean manifestation.  If the resultant entity's primary
   key is serializable then it is returned to the caller, otherwise null is
   returned

   @param inXMLString - The XML being converted to entity instances
   @return Serializable primary key of the outermost object created
   @throws java.rmi.RemoteException
   @roseuid 4082EE270201
    */
   public Serializable convertToEntity(String inXMLString) throws EntityException,RemoteException;

   public Serializable convertToFullEntity(String inXMLString) throws EntityException, RemoteException;

   public boolean ping() throws RemoteException;
}
