package org.quintet.chambers;

import javax.ejb.EJBHome;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import java.io.InputStream;

/**

 * <code>EntityFactoryHome</code> is the home interface for <code>EntityFactory</code>.
 * <code>EntityFactory</code> is stateful; you must supply a <i>blueprints file</i> when
 * creating an <code>EntityFactory</code> instance.
 * @author PThaggard
 * @version 2.0
 * @since October, 2001
 */
public interface EntityFactoryHome extends EJBHome
{

   /**
    * Create an <code>EntityFactory</code> instance that utilizes the <i>blueprints</i>
    * found in the <code>blueprintsStream</code> <code>InputStream</code>
    * @return org.quintet.chambers.EntityFactory
    * @throws java.rmi.RemoteException
    * @throws javax.ejb.CreateException
    * @roseuid 407FF4F800B4
    */
   public EntityFactory create(InputStream blueprintsStream) throws RemoteException, CreateException;

   /**
    * Create an <code>EntityFactory</code> that utilizes the <i>blueprints</i> found in
    * the file at <code>blueprintsURL</code>
    * @param blueprintsURL String -- The URL of a blueprints file
    * @throws RemoteException
    * @throws CreateException
    * @return EntityFactory
    */
   public EntityFactory create(String blueprintsURL) throws RemoteException, CreateException;
}
