package org.quintet.chambers;

import javax.ejb.EJBLocalObject;
import java.io.Serializable;
import java.util.Collection;

/**
  *<code>EntityFactory</code> is a session bean front-end to
  *<code>Monk</code>.  <code>EntityFactory</code> lives within a WebLogic
  *JVM, therefore it is capable of constructing local entity beans, which
  *<code>Monk</code> is unable to do on its own.<br>
  *<code>EntityFactory</code> already has all the necessary plumbing to perform
  * <i>dynamic EJB-QL queries</i> against its entity domains, so that functionality
  * is also offered (via the <code>dynaQuery</code> method)<br>
  * @author Peter Thaggard
  * @since Quintet version 2002.1
 */
public interface EntityFactoryLocal
    extends EJBLocalObject {

  /**
   <code>convertToEntity</code> accepts a String containing XML and converts its
      contents to an entity bean manifestation.

      @param inXMLString
      @throws java.rmi.RemoteException
      @roseuid 4082EE270201
   */
  public Serializable convertToEntity(String inXMLString) throws
      EntityException;

  /**
   * Execute the query defined by <code>inEJBQLString</code> against
   * the
   * @param inEJBQLString String
   * @return Collection
   * @throws EntityException
   */
  public Collection dynaQuery(String inEJBQLString) throws EntityException;
}
