package org.quintet.chambers;

/**
 * An <code>EntityRetriever</code> is responsible for retrieving a fully-formed 
 * <i>entity</i> from its concomitant <i>primary key</i>.  Entities are frequently
 * presented in a lightweight manner in order to minimize network traffic; for example,
 * a "request" entity might be persisted to a database and then only the request's unique identifier,
 * or primary key, needs to be sent to a <code>Supplier</code>. A <code>Supplier</code> 
 * would subsequently use an <code>EntityRetriever</code>s to retrieve the complete persisted
 * request from the database. 
  *@author Peter Thaggard
  *@version 1.0
  *@since Quintet 2004.1
**/

public interface EntityRetriever{
	
	/**
	 * "Retrieve" the request whose unique identifier is <code>pk</code> and return
	 * the fully-formed request to the caller.  The exact nature of the <code>pk</code>
	 * and its concomitant fully-formed object are determined by both the concrete
	 * <code>EntityRetriever</code> implementation and by its instances.
	 * @param pk
	 * @return
	 * @throws RetrieverException
	 */
	public Object retrieve(Object pk) throws RetrieverException;
}
