package org.quintet.chambers;

import java.util.Map;
import java.util.Collection;
import java.util.ArrayList;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * <code>ParameterizedSQLQueryRetriever</code> retrieves a batch of SQL statements from JNDI,
* replaces their parameters (symbols surrounded by #) with actual values taken from <code>inParameters</code>,
* and then returns the SQL statements as a String array
 */
public class ParameterizedSQLQueryRetriever implements EntityRetriever{

  private static final Logger log = Logger.getLogger("org.quintet.chambers");
  public void closeResources(InitialContext ic){
    if(ic != null){
      try{
        ic.close();
      }catch(Throwable eatIt){}
    }
  }
  /**
   * Fetch a set of queries (env-entry values starting with select-), replace their
   * parameters with actual values taken from inParameters, return the queries as a String[]
   * @param inParameters Object Map of parameter-name=>parameter-value
   * @throws RetrieverException
   * @return Object String[]
   */
  public Object retrieve(Object inParameters) throws RetrieverException{
    Map parameters = null;
    boolean replaceParameters = true;
    if(inParameters == null){
      replaceParameters = false;
    }else{
      if(inParameters instanceof java.util.Map){
        parameters = (Map)inParameters;
      }else{
        replaceParameters = false;
      }
    }
    InitialContext ic = null;
    try {
      ic = new InitialContext();
    }
    catch (NamingException ne) {
      System.out.println(
          "unable to obtain initial " +
          "context, returning zero SELECTS from deployment descriptor");
      closeResources(ic);
      return null;
    }

    boolean someMore = true;
    int selectCnt = 0;
    Collection allSelects = new ArrayList();
    while(someMore == true){
      String currSelectName = "java:comp/env/select-" + selectCnt++;
      try{
        String currSelect = (String)ic.lookup(currSelectName);
        if(replaceParameters == false){
          allSelects.add(currSelect);
        }else{
          allSelects.add(replaceParameters(currSelect, parameters));
        }
      }catch(NamingException ne){
        someMore = false;
      }
    }
    closeResources(ic);
    return (String[])allSelects.toArray(new String[1]);
  }

  private String replaceParameters(String parameterized, Map parameters) throws RetrieverException{
    int hashIndex = -1;
    hashIndex = parameterized.indexOf("#");
    while(hashIndex != -1)
    {
        //search for the next # sign. The string between these 2
        //hashes has to be replaced by a value from the object map
        int endHashIndex = parameterized.indexOf("#", hashIndex + 1);
        if(endHashIndex != -1)
        {
            String valToBeReplaced =
                parameterized.substring(hashIndex + 1, endHashIndex);

            //get the value for this string
            //from the message map
            if(valToBeReplaced != null)
            {
                String repStr =
                    (String)parameters.get(valToBeReplaced);
                if(repStr == null || repStr.equals(""))
                {
                    throw new RetrieverException(
                        ":Value not provided for query parameter: " +
                        repStr);
                }
                //now replace all occurrances of #valTobeReplaced#
                //with the value from hashmap
                parameterized = parameterized.replaceAll(
                    "#" +valToBeReplaced +"#", repStr);
            }
        }
        hashIndex = parameterized.indexOf("#");
    }
    return parameterized;

  }
}

