package org.quintet.chambers;


import org.quintet.QuintetException;

/**
 * Objects implementing the <code>Passivator</code> interface agree to provide
 * facilities for <i>passivating</i> (that is, swapping out of memory and into/onto
 * some sort of persistent storage facility) and <i>activating</i> (that is,
 * loading from persistent storage back into memory) objects of the type specified
 * by the &lt;SubjectType&gt; formal generic argument.
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>Company: 6th Sense Analytics</p>
 *
 * @author Peter Thaggard
 * @version 1.0
 */
public interface Passivator<SubjectType> {

  /**
   * Swap <code>subject</code> to persistent storage, return the <code>Long</code>
   * by which <code>subject</code> can subsequently be retrieved back from
   * persistent storage.
   *
   * @param subject SubjectType
   * @return Long
   * @throws QuintetException
   */
  public Long passivate(SubjectType subject) throws QuintetException;

  /**
   * Load the object located at <code>passivatedID</code> back into memory
   * and then return the object to the caller.
   * @param passivatedID Long
   * @return SubjectType
   * @throws QuintetException
   */
  public SubjectType activate(Long passivatedID) throws QuintetException;


}
