package org.quintet.ellington;

import java.util.Collection;
import java.util.Map;
import java.util.HashMap;

import java.util.logging.Logger;
import java.util.logging.Level;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.quintet.QuintetException;

/**
 * <code>JNDIArranger</code> "arranges" an object (i.e., helps to configure it)
 * by reading a set of property values from JNDI.  The properties to be read
 * are provided by the caller, and both the property names and their values
 * are returned in a <code>Map</code>.<p>
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: 6th Sense Analytics</p>
 *
 * @author Peter Thaggard
 * @version 1.0
 */
public class JNDIArranger extends Arranger {



  public JNDIArranger() {
    super();
  }

  /**
   * Search JNDI to find values for the properties mentioned in <code>propertyNames</code>.
   * Return a <code>Map&lt;String,Object&gt;</code> where the property names are
   * the keys and the property values are the values.  If a requested property
   * is not found then return its value as <code>null</code>.<br>
   * @param propertyNames String[]
   * @return Map&lt;String,Object&gt;
   * @throws QuintetException
   */
  public Map<String,Object> arrange(String[] propertyNames) throws QuintetException {
    InitialContext ic = null;
    try{
      ic = new InitialContext();
    }catch(NamingException ne){
      String err = "Unable to obtain a JNDI InitialContext.  This class only "+
          "works within the JVM of a J2EE app server.";
      log.log(Level.SEVERE,err,ne);
      throw new QuintetException(err,ne);
    }

    Map<String,Object> answers = new HashMap<String,Object>();

    for(String rawPropertyName:propertyNames){
      String propertyName = "java:comp/env/" + rawPropertyName;
      Object value = null;
      try{
        value = ic.lookup(propertyName);
      }catch(NamingException ne){
        value = null;
      }
      answers.put(rawPropertyName,value);
    }

    return answers;
  }
}
