package org.quintet.miles;

import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * <code>ActivatorFactory</code> is an exceedingly crude device for instantiating
 * concrete <code>Activator</code> subclasses; it is essentially an application-level
 * version of <code>Class.forName</code>.
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>Company: 6th Sense Analytics</p>
 *
 * @author Peter Thaggard
 * @version 2006.1.0
 */
public class ActivatorFactory {

  private static  final Logger log = Logger.getLogger("org.quintet.miles");

  public ActivatorFactory() {
  }

  public static Activator create(String activatorClassName, Class inSubject) throws IllegalStateException{
    //PRECONDITIONS:
      //PRECONDITION 1:  activatorClassName IS NOT NULL
      if(null == activatorClassName){
        String ixnay = "you must supply a non-null activator class name";
        log.log(Level.WARNING,ixnay);
        throw new IllegalStateException(ixnay);
      }


    Activator answer = null;
    Class answerClass = null;
    try {
      answerClass = Class.forName(activatorClassName);
    }
    catch (ClassNotFoundException ex) {
      String err = "ClassNotFoundException from Class.forName(\"" + activatorClassName + "\")";
      log.log(Level.SEVERE,err,ex);
      throw new IllegalStateException(err,ex);
    }

    try {
      answer = (Activator)answerClass.newInstance();
      answer.setSubject(inSubject);
    }
    catch (IllegalAccessException ex1) {
      String err = "IllegalAccessException when invoking '" + answerClass.getName() + ".newInstance()'";
      log.log(Level.SEVERE,err,ex1);
      throw new IllegalStateException(err,ex1);
    }
    catch (InstantiationException ex1) {
      String err = "InstantiationException when invoking '" + answerClass.getName() + ".newInstance()'";
      log.log(Level.SEVERE,err,ex1);
      throw new IllegalStateException(err,ex1);
    }

    if(!(answer instanceof Activator)){
      String no = "Expected an Activator subclass, but you supplied a '" + answer.getClass().getName() + "'";
      log.log(Level.WARNING,no);
      throw new IllegalStateException(no);
    }

    return (Activator)answer;
  }
}
