package org.quintet.miles;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.StringTokenizer;

import java.util.logging.Logger;
import java.util.logging.Level;

public class DN implements Serializable,Cloneable{

  	
	/**
	 * @clientRole nodes
	 * @directed true
	 * @link aggregationByValue
	 * @supplierCardinality 1..*
	 */
	
  	private org.quintet.miles.DNNode lnkDNNode;

private  static final Logger log = Logger.getLogger("org.quintet.miles");

  private Queue<DNNode<String,String>> nodes = new LinkedList<DNNode<String,String>> ();

  /**
   * Return a comma-separated string of the <code>LHS</code>s comprising this
   * <code>DN</code>
   * @return String
   */
  public String getNodeStructure(){
    int nodeIdx = 0;
    StringBuffer answer = new StringBuffer();
    List<DNNode<String, String> > nodesList = (List<DNNode<String, String> >)nodes;
    for(DNNode<String, String> node:nodes){
      String lhs = node.LHS;
      answer.append(lhs);
      if(++nodeIdx < nodes.size())
        answer.append(',');
    }
    return answer.toString();
  }

  /**
   * Private constructor, used exclusively by {@link org.quintet.miles.DN#clone()}
   */
  private DN(){

  }

  /**
   * Construct a new <code>DN</code> that implements the DN described by
   * <code>stringDN</code>
   * @param stringDN String
   * @throws AggregationException
   */
  public DN(String stringDN) throws AggregationException {
    if (null == stringDN || stringDN.length() == 0) {
      String err = "Creating an empty DN from your null/empty string";
      log.log(Level.WARNING,err);
      return;
    }

    StringTokenizer st = new StringTokenizer(stringDN, ",");
    while (st.hasMoreTokens()) {
      String currToken = st.nextToken();
      int eqPos = currToken.indexOf("=");
      if (eqPos < 0) {
        String err = "Syntax error in DN '" + stringDN + "' -- missing '='";
        log.log(Level.SEVERE,err);
        throw new AggregationException(err);
      }

      if (eqPos == 0) {
        String err = "Syntax error in DN '" + stringDN +
            "' -- missing LHS (subject Class)";
        log.log(Level.SEVERE,err);
        throw new AggregationException(err);
      }

      String subjectClass = currToken.substring(0, eqPos).trim();

      String subjectValue = currToken.substring(eqPos + 1).trim();

      DNNode<String, String>
          newNode = new DNNode<String, String> (subjectClass, subjectValue);

      nodes.add(newNode);
    }
    return;
  }

  private static Map<String,String> rollupTokens = new java.util.HashMap<String,String>();

  static{
    rollupTokens.put("week","day");
    rollupTokens.put("month","day");
    rollupTokens.put("year","day");
  }

  /**
   * If <code>token</code> is considered a "rollup token" (i.e., it is something
   * like "week" that does not have a direct correlation to an aggregation) then
   * return the "un-rolled-up" version of token; for example, if the <code>token</code>
   * is "week" then this method will return "day".
   * If <code>token</code> is not a rollup token then it is returned verbatim to
   * the caller.
   * @param token String
   * @return String
   */
  public static String translateRollupToken(String token){
    String rollUpToken = rollupTokens.get(token);
    if(null == rollUpToken)
      return token;
    else
      return rollUpToken;
  }

  public static boolean isRollupToken(String token){
    String rollupToken = rollupTokens.get(token);
    return rollupToken != null;
  }


  public Queue<DNNode<String, String>> getNodes() {
    return nodes;
  }

  public boolean offer(DNNode<String, String> object) {
    return nodes.offer(object);
  }

  public DNNode<String, String> poll() {
    return nodes.poll();
  }

  public DNNode<String, String> remove() {
    return getNodes().remove();
  }

  public DNNode<String, String> peek() {
    return getNodes().peek();
  }

  public DNNode<String, String> element() {
    return getNodes().element();
  }

  public static boolean in(String lhs, DNNode<String, String> rhs) {
    String membershipTest = rhs.RHS;
    StringTokenizer st = new StringTokenizer(membershipTest, "|");
    while (st.hasMoreTokens()) {
      String currMembershipTest = st.nextToken();
      if (currMembershipTest.equals(lhs)) {
        return true;
      }
    }
    return false;
  }

  public String toString(){
    //log.log(Level.SEVERE,"Quintet Version 20060525");
    StringBuffer answer = new StringBuffer();
    for(DNNode<String, String> currNode:getNodes()){
      answer.append(currNode.LHS + '=' + currNode.RHS + ',');
    }
    return answer.toString();
  }

  public Object clone(){
    DN cloned = new DN();
    for(DNNode srcNode:getNodes()){
      DNNode<String,String> newNode = (DNNode)srcNode.clone();
      cloned.getNodes().add(newNode);
    }
    return cloned;
  }

  /**
   * "Plug holes" in <code>originalRange</code> by ensuring that the resulting <code>Range</code>
   * contains a "slot" for every RHS token explicitly mentioned.  For example, given: <br>
   * <li>The original query is "<code>uid=201,day=01/01/2006|01/02/2006|01/03/2006</code>"
   * <li><code>uid 201</code> had no activity on 02/02/2006
   * <br>
   * this method will note the absence of a "slot" for 01/02/2006 and will "fill the gap" by inserting
   * a slot containing the value <code>zero</code>
   * @param originalRange Range
   * @return Range
   * @throws AggregationException
   */
  public Range fillInHoles(Range originalRange) throws AggregationException{
      for(DNNode<String,String> node:getNodes()){
          String rhs = node.RHS;
          //for each OR-separated rhs value, look for a lot in the originalRange
          StringTokenizer lex = new StringTokenizer(rhs,"|");

      }
      return originalRange;
  }

}
