package org.quintet.miles;

import java.util.Iterator;

/**
 * <code>NullIterator</code> is a very special <code>Iterator</code>; its <code>hasNext()</code>
 * and <code>next</code> methods <b>always</b> return null.  <code>NullIterator</code> is useful
 * in situations like <code>Range.iterator()</code>, which wraps <code>List.iterator</code>; if
 * the <code>Range</code>'s underlying <code>members</code> attribute is null, then <code>Range.iterator()</code>
 * would normally return <code>null</code>, but this frequently results in NPEs in calling clients.
 * By returning a <code>NullIterator</code> instead of <code>null</code>, methods like <code>Range.iterator()</code>
 * preserve their clients' perceived contractual obligations.
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
class NullIterator implements Iterator {
    /**
     * hasNext
     *
     * @return boolean
     * @todo Implement this java.util.Iterator method
     */
    public boolean hasNext() {
        return false;
    }

    /**
     * next
     *
     * @return Object
     * @todo Implement this java.util.Iterator method
     */
    public Object next() {
        return null;
    }

    /**
     * remove
     *
     * @todo Implement this java.util.Iterator method
     */
    public void remove() {
    }
}
