package org.quintet.miles;
import java.util.logging.Logger;
import java.util.logging.Level;

public class RangeFactory {
  private static final Logger log = Logger.getLogger("org.quintet.miles");

  public RangeFactory() {
  }
  public static Range createRange(String rangeClassName) throws AggregationException{
    if(rangeClassName == null || rangeClassName.length() == 0){
      String err = "Unable to process null or empty class names";
      log.log(Level.SEVERE,err);
      throw new AggregationException(err);
    }
    Class rangeClass = null;
    try {
      rangeClass = Class.forName(rangeClassName);
    }
    catch (ClassNotFoundException ex) {
      String err = "ClassNotFoundException when trying to load class named '" + rangeClassName + "'";
      log.log(Level.SEVERE,err,ex);
      throw new AggregationException(err,ex);
    }

    if(! (Range.class.isAssignableFrom(rangeClass))){
      String err = rangeClassName + " must be a subclass of org.quintet.miles.Range";
      log.log(Level.SEVERE,err);
      throw new AggregationException(err);
    }
    Range newInstance = null;
    try {
      newInstance = (Range)rangeClass.newInstance();
      return newInstance;
    }
    catch (IllegalAccessException ex1) {
      String err = "IllegalAccessException when constructing a new " + rangeClassName ;
      log.log(Level.SEVERE,err,ex1);
      throw new AggregationException(err,ex1);
    }
    catch (InstantiationException ex1) {
      String err = "InstantiationException when constructing a new " + rangeClassName ;
      log.log(Level.SEVERE,err,ex1);
      throw new AggregationException(err,ex1);

    }


  }
}
