package org.quintet.monk;

import java.util.logging.Logger;
import java.util.logging.Level;

/**
An <code>Attribute</code> houses an attribute's XML-namespace name (e.g.,
dealer-id), its java-namespace name (e.g., dealerID), and its java type (e.g.,
java.lang.String).
 */
public class Attribute
{
  private static final Logger log = Logger.getLogger("org.quintet.monk");
   /**
   The attribute's name in the XML namespace
    */
   private String xmlName = null;

   /**
   The attribute's name in the java namespace
    */
   private String javaName = null;

   /**
   The name of this <code>Attribute</code>'s java class.
    */
   private String javaClassName = null;
   private String setterName = null;

   /**
   Indicates if this <code>Attribute</code> is a hard coded (i.e., literal) value.
    */
   private String literal = "false";

   /**
   The <code>Class</code> to which the attribute belongs; for example, a
   <code>dealerID</code> attribute's <code>javaClass</code> might be
   <code>java.lang.String</code>
    */
   private Class javaClass = null;

   /**
   @roseuid 40852A120317
    */
   public Attribute()
   {

   }

   /**
   Access method for the xmlName property.

   @return   the current value of the xmlName property
    */
   public String getXmlName()
   {
    return xmlName;
   }

   /**
   Sets the value of the xmlName property.

   @param aXmlName the new value of the xmlName property
    */
   public void setXmlName(String aXmlName)
   {
    xmlName = aXmlName;
   }

   /**
   Access method for the javaName property.

   @return   the current value of the javaName property
    */
   public String getJavaName()
   {
    return javaName;
   }

   /**
   Sets the value of the javaName property.

   @param aJavaName the new value of the javaName property
    */
   public void setJavaName(String aJavaName)
   {
    javaName = aJavaName;
   }

   /**
   Access method for the javaClassName property.

   @return   the current value of the javaClassName property
    */
   public String getJavaClassName()
   {
    return javaClassName;
   }

   /**
   Sets the value of the javaClassName property.

   @param aJavaClassName the new value of the javaClassName property
    */
   public void setJavaClassName(String aJavaClassName)
   {
    javaClassName = aJavaClassName;
    try {
      Class it = Class.forName(javaClassName);
      setJavaClass(it);
    }
    catch (ClassNotFoundException CNFE) {
      String err = "Class '" + aJavaClassName + "' not found";
      log.log(Level.SEVERE,err);
      throw new IllegalStateException(err);
    }
   }

   /**
   Access method for the setterName property.

   @return   the current value of the setterName property
    */
   public String getSetterName()
   {
    return setterName;
   }

   /**
   Sets the value of the setterName property.

   @param aSetterName the new value of the setterName property
    */
   public void setSetterName(String aSetterName)
   {
    setterName = aSetterName;
   }

   /**
   Access method for the literal property.

   @return   the current value of the literal property
    */
   public String getLiteral()
   {
      return literal;
   }

   /**
   Sets the value of the literal property.

   @param aLiteral the new value of the literal property
    */
   public void setLiteral(String aLiteral)
   {
      literal = aLiteral;
      setJavaClass(literal.getClass());
   }

   /**
   Access method for the javaClass property.
   @return   the current value of the javaClass property
   @roseuid 40E9B06702E7
    */
   public Class getJavaClass()
   {
    return javaClass;
   }

   /**
   Sets the value of the javaClass property.
   @param aJavaClass the new value of the javaClass property
   @roseuid 40E9B0670305
    */
   public void setJavaClass(Class aJavaClass)
   {
    javaClass = aJavaClass;
   }

   /**
   Returns true if and only if this <code>Attribute</code> represents a literal
   value
   @return boolean
   @roseuid 42A5C03C00F2
    */
   public boolean isLiteral()
   {
    return literal != null && literal.equalsIgnoreCase("true");
   }

   private Boolean mandatory = new Boolean("false");
   public Boolean isMandatory(){
     return getMandatory();
   }
   public Boolean getMandatory(){
     return mandatory;
   }
   public void setMandatory(Boolean v){
     mandatory = v;
   }
}
