package org.quintet.monk;

import java.lang.reflect.Method;
import java.io.FileNotFoundException;
import java.util.Map;
import java.io.InputStream;
import java.io.FileInputStream;
import java.util.Iterator;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.Set;
import java.util.Collection;
import java.lang.reflect.Modifier;

public class Blueprints implements Map
{
   private Map impl = new java.util.HashMap ();

   /**
   <code>Blueprints</code> bootstraps itself via a primitive <code>Monk</code> (primitive in the sense that it contains no blueprints, only a basic <code>classMap</code>
    */
   private static Monk bootstrapFactory = null;
   protected static final Logger log = Logger.getLogger("org.quintet.monk");

   /**
   @roseuid 426925D80216
    */
   public Blueprints()
   {

   }

   static
   {
    try{
      bootstrapFactory = Monk.createMonk();
    }catch(MonkException ofe){
      String ow = "Unable to create bootstrap Monk";
      log.log(Level.SEVERE,ow,ofe);
      throw new java.lang.Error(ow,ofe);
    }
   }

   /**
   Access method for the bootstrapFactory property.

   @return   the current value of the bootstrapFactory property
    */
   public static Monk getBootstrapFactory()
   {
      return bootstrapFactory;
   }

   /**
   Sets the value of the bootstrapFactory property.

   @param aBootstrapFactory the new value of the bootstrapFactory property
    */
   public static void setBootstrapFactory(Monk aBootstrapFactory)
   {
      bootstrapFactory = aBootstrapFactory;
   }

   /**
   @return java.util.Map
   @roseuid 4269231D0243
    */
   protected Map getImpl()
   {
    if (impl == null) {
      impl = new java.util.HashMap();
    }
    return impl;
   }

   protected void setImpl(Map inImpl){
     impl = inImpl;
   }
   /**
   @param blueprintsFile
   @return org.quintet.monk.Blueprints
   @throws org.quintet.monk.MonkException
   @roseuid 426923A400A2
    */
   public static  Blueprints buildBlueprints(InputStream blueprintsFile) throws MonkException
   {
    Blueprints blueprints = new Blueprints();
      //turn the blueprintsFile's contents into some
      //java objects (instances of ClassMetaData, Builder, and Attribute)
      Collection blueprintsCollection = (Collection) bootstrapFactory.create(blueprintsFile);
      if (blueprintsCollection == null || blueprintsCollection.size() == 0) {
        String err = ":no blueprints available";
        log.log(Level.SEVERE,err);
        throw new MonkException(err);
      }
      for (Iterator blueprintIter = blueprintsCollection.iterator();
           blueprintIter.hasNext(); ) {
        ClassMetaData blueprint = (ClassMetaData) blueprintIter.next();
        String xmlName = blueprint.getXmlName();
        Builder b = blueprint.getBuilder();
        Class[] argClasses = b.getFormalArgumentClasses();
        Object targetObj = null;
        if (b.isCtor()) {
          if (!b.getIsDeferred()) {
            try {
              Class targetClass = Class.forName(b.getTargetName());
              b.setTarget(targetClass);
            }
            catch (ClassNotFoundException CNFE) {
              String err =
                  ":Unable to find class mentioned in blueprints:" +
                  b.getTargetName() + ":" + CNFE;
              log.log(Level.SEVERE,err);
              throw new MonkException(err, CNFE);
            }
          }
        }
        else {
            //this is a non-EJB factory class
            Class factoryClass = null;
            if (b.getIsDeferred() == false) {
              try {
                factoryClass = Class.forName(b.getTargetName());
                //if the class is abstract then we obviously cannot instantiate it.
                //However, the blueprints may have specified a static method of an
                //abstract class, so we have to deal with that possibility
                int factoryClassModifiers = factoryClass.getModifiers();
                if(Modifier.isAbstract(factoryClassModifiers)){
                  log.info(b.getTargetName() + " is abstract, therefore its method must be static");
                  b.setAbstractFactoryClass(true);
                  b.setStaticMethod(true);
                  targetObj = factoryClass;
                }else{
                  targetObj = factoryClass.newInstance();
                }
              }
              catch (IllegalAccessException ex1) {
                String err = ":ERROR:IllegalAccessException when "
                    + "trying to instantiate a " + b.getTargetName() + ":" +
                    ex1;
                log.log(Level.SEVERE,err);
                throw new MonkException(err, ex1);
              }
              catch (InstantiationException ex1) {
                String err = ":ERROR:InstantiationException when "
                    + "trying to instantiate a " + b.getTargetName() + ":" +
                    ex1;
                log.log(Level.SEVERE,err);
                throw new MonkException(err, ex1);
              }
              catch (ClassNotFoundException ex) {
                String err =
                    ":ERROR:unable to find factory class " + b.getTargetName() +
                    ":" + ex;
                log.log(Level.SEVERE,err);
                throw new MonkException(err, ex);
              }
              b.setTarget(targetObj);
              try {
                if(b.getAbstractFactoryClass() == true){
                  Method ctor = ((Class)targetObj).getMethod(b.getFactoryMethodName(),argClasses);
                  b.setFactoryMethod(ctor);
                  b.setStaticMethod(true); //it has to be static
                }else{
                  Method ctor = targetObj.getClass().getMethod(b.
                      getFactoryMethodName(), argClasses);
                  b.setFactoryMethod(ctor);
                  if(Modifier.isStatic(ctor.getModifiers()))
                    b.setStaticMethod(true);
                }
              }
              catch (SecurityException ex2) {
                String err = ":unable to find method named " +
                    b.getFactoryMethodName() + " in class " +
                    targetObj.getClass() +
                    ":" + ex2;
                log.log(Level.SEVERE,err);
                log.log(Level.SEVERE,ex2.toString());
                throw new MonkException(err);
              }
              catch (NoSuchMethodException ex2) {
                String err = ":unable to find method named " +
                    b.getFactoryMethodName() + " in class " +
                    targetObj.getClass() +
                    ":" + ex2;
                log.log(Level.SEVERE,err);
                log.log(Level.SEVERE,ex2.toString());
                throw new MonkException(err);
              }
            }

        }

        log.log(Level.FINE,"blueprint entry:  " + xmlName + "==>" + blueprint.getSubjectName());
        blueprints.put(xmlName, blueprint);
      }
    return blueprints;
   }

   /**
   @param fileName
   @return org.quintet.monk.Blueprints
   @throws org.quintet.monk.MonkException
   @roseuid 4269249A016E
    */
   public static Blueprints buildBlueprints(String fileName) throws MonkException
   {
    FileInputStream fis = null;
    try {
      fis = new FileInputStream(fileName);
      Blueprints answer = Blueprints.buildBlueprints(fis);
      return answer;
    }
    catch (FileNotFoundException ex) {
      String err = "unable to open blueprints file named '" + fileName + "'";
      log.log(Level.SEVERE,err, ex);
      throw new MonkException(err, ex);
    }
    finally {
      if (fis != null) {
        try {
          fis.close();
        }
        catch (Throwable t) {}
      }
    }
   }

   /**
   @param forClass
   @param fileName
   @return org.quintet.monk.Blueprints
   @throws org.quintet.monk.MonkException
   @roseuid 426924B601FB
    */
   public static Blueprints buildBlueprints(Class forClass, String fileName) throws MonkException
   {
    InputStream str = null;
    str = forClass.getResourceAsStream(fileName);
    return Blueprints.buildBlueprints(str);
   }

   /**
   @return int
   @roseuid 426925D80234
    */
   public int size()
   {
    return getImpl().size();
   }

   /**
   @return boolean
   @roseuid 426925D8023E
    */
   public boolean isEmpty()
   {
    return getImpl().isEmpty();
   }

   /**
   @param arg0
   @return boolean
   @roseuid 426925D80248
    */
   public boolean containsKey(Object arg0)
   {
    return getImpl().containsKey(arg0);
   }

   /**
   @param arg0
   @return boolean
   @roseuid 426925D80266
    */
   public boolean containsValue(Object arg0)
   {
    return getImpl().containsValue(arg0);
   }

   /**
   @param arg0
   @return java.lang.Object
   @roseuid 426925D8027A
    */
   public Object get(Object arg0)
   {
    return getImpl().get(arg0);
   }

   /**
   @param arg0
   @param arg1
   @return java.lang.Object
   @roseuid 426925D80298
    */
   public Object put(Object arg0, Object arg1)
   {
    return getImpl().put(arg0, arg1);
   }

   /**
   @param arg0
   @return java.lang.Object
   @roseuid 426925D802C0
    */
   public Object remove(Object arg0)
   {
    return getImpl().remove(arg0);
   }

   /**
   @param arg0
   @roseuid 426925D802DE
    */
   public void putAll(Map arg0)
   {
    getImpl().putAll(arg0);
   }

   /**
   @roseuid 426925D802F2
    */
   public void clear()
   {
    getImpl().clear();
   }

   /**
   @return java.util.Set
   @roseuid 426925D802FC
    */
   public Set keySet()
   {
    return getImpl().keySet();
   }

   /**
   @return java.util.Collection
   @roseuid 426925D80306
    */
   public Collection values()
   {
    return getImpl().values();
   }

   /**
   @return java.util.Set
   @roseuid 426925D8031A
    */
   public Set entrySet()
   {
    return getImpl().entrySet();
   }

   /**
   @param arg0
   @return boolean
   @roseuid 426925D80324
    */
   public boolean equals(Object arg0)
   {
    return getImpl().equals(arg0);
   }

   /**
   @return int
   @roseuid 426925D80342
    */
   public int hashCode()
   {
    return getImpl().hashCode();
   }


}
