package org.quintet.monk;

import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Iterator;
import java.util.Arrays;
import java.beans.PropertyDescriptor;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.lang.reflect.InvocationTargetException;
import java.io.OutputStream;
import java.io.IOException;
import java.io.PrintWriter;

/**
 Factory class to create XML representations of java object lattices.
 */
public class XMLFactory {
  private static String wholeLottaTabs =
      "\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t";
  private static final Logger log = Logger.getLogger("org.quintet.monk");

  /**
      @roseuid 425A86D20343
   */
  public XMLFactory() {

  }

  /**
      Access method for the wholeLottaTabs property.

      @return   the current value of the wholeLottaTabs property
   */
  public static String getWholeLottaTabs() {
    return wholeLottaTabs;
  }

  /**
      Sets the value of the wholeLottaTabs property.

      @param aWholeLottaTabs the new value of the wholeLottaTabs property
   */
  public static void setWholeLottaTabs(String aWholeLottaTabs) {
    wholeLottaTabs = aWholeLottaTabs;
  }

  /**
      Access method for the log property.

      @return   the current value of the log property
   */
  public static Logger getLog() {
    return log;
  }

  /**
      Create an XML manifestation of <code>inObject</code>.  This method
   makes an effort to process most kinds of objects, including regular class
      instances,
      arrays, and collections.  However, it only works with homogenous collections and
      arrays;
      it throws <code>XMLInfoException</code> if it encounters a heterogenous
      collection or array.
   This method considers two objects to be homegenous if they both resolve to the
      same XMLInfo
   object.  So, for example, if Loan and CompositeLoan both use the LoanXMLInfo
      XMLInfo object,
   then a collection containing both Loans and CompositeLoan's is considered
      homegenous.
      @param java.lang.Object inObject The object being manifested as XML
      @param inObject
      @exception XMLInfoException
   @return String containing <code>inObject</code>'s XML manifestation@throws
      org.quintet.monk.XMLInfoException
      @roseuid 425A86D2034D
   */
  public static String createXML(java.lang.Object inObject) throws
      XMLInfoException {
    return XMLFactory.createXML(inObject, 0);
  }

  public static void createXML(Object inObject, OutputStream dest) throws XMLInfoException{
    String prelim = XMLFactory.createXML(inObject,0);
    PrintWriter pw = new PrintWriter(dest);
    pw.println(prelim);
    pw.close();
  }
  /**
      @param inObject
      @param indentLevel
      @return java.lang.String
      @throws org.quintet.monk.XMLInfoException
      @roseuid 425A86D203B2
   */
  private static String createXML(java.lang.Object inObject, int indentLevel) throws
      XMLInfoException {
    if (inObject == null) {
      return null;
    }
    //if inObject is a map, collection or array, then recursively call
    //createXML for every object therein.
    XMLInfo outermost = null;
    outermost = XMLIntrospector.getXMLInfo(inObject.getClass());

    if (outermost.isCollection() && (isMap(inObject) || isCollection(inObject) || isArray(inObject))) {
      Iterator memberIter = null;
      XMLInfo previousEncounter = null;

      if (isArray(inObject)) {
        memberIter = ( (Collection) Arrays.asList( (Object[]) inObject)).
            iterator();
      }
      else if(isCollection(inObject)){
        memberIter = ( (Collection) inObject).iterator();
      }else{
        //it must be a map
        memberIter = ((java.util.Map)inObject).entrySet().iterator();
      }
      StringBuffer answer = new StringBuffer();
      //we need some sort of outer shell XML element to contain
      //the collection's XML manifestations.  Since inObject's
      //members MUST be homogenous, we can use the first member's
      //XMLInfo's elementName as the name of the collection
      previousEncounter = XMLIntrospector.getXMLInfo(inObject.getClass());
      if (previousEncounter == null) {
        String err = "Unable to get XMLInfo for first member of inObject";
        throw new XMLInfoException(err);
      }

      answer.append(tabs(indentLevel) + "<" + outermost.getElementName() +
                    "-collection\n");
      Collection attributes = outermost.getAttributes();
      if (attributes.size() > 0) {
        answer.append("\n");
      }
      for (Iterator attrIter = attributes.iterator(); attrIter.hasNext(); ) {
        PropertyDescriptor currAttr = (PropertyDescriptor) attrIter.next();
        String attrName = currAttr.getName();
        Method getter = currAttr.getReadMethod();
        if(getter != null){
          Object attrValue = null;
          try {
            attrValue = getter.invoke(inObject, (Object[])null);
          }
          catch (InvocationTargetException ex) {
            throw new XMLInfoException("InvocationTargetException", ex);
          }
          catch (IllegalArgumentException ex) {
            throw new XMLInfoException("IllegalArgumentException", ex);
          }
          catch (IllegalAccessException ex) {
            throw new XMLInfoException("IllegalAccessException", ex);
          }
          if (attrValue != null) {
            String attrStr = attrValue.toString();
            answer.append(tabs(indentLevel + 1) + attrName + " = " + "\"" +
                          attrStr + "\"" + "\n");
          }
        }
      }
      answer.append(">\n");

      previousEncounter = null;
      while (memberIter.hasNext()) {
        Object currMember = memberIter.next();
        XMLInfo currInfo = XMLIntrospector.getXMLInfo(currMember.getClass());
        if (currInfo == null) {
          String err = "Unable to get XMLInfo for member of type " +
              currInfo.getClass().getName();
          throw new XMLInfoException(err);
        }
       /** if (previousEncounter != null && (currInfo.equals(previousEncounter)) == false) {
          String err = "Unable to process heterogenous collections/arrays.  ";
          throw new XMLInfoException(err);
        }**/

        answer.append(createXML(currMember, indentLevel + 1));
        previousEncounter = currInfo;
      }

      answer.append(tabs(indentLevel) + "</" + outermost.getElementName() +
                    "-collection>\n");
      return answer.toString();
    }
    else {
      //use inObject's concomitant XMLInfo's element name


      String topName = outermost.getElementName();
      if (topName == null || topName.length() == 0) {
        throw new XMLInfoException(inObject.getClass().getName() +
                                   " lacks an elementName.");
      }
      return createXML(inObject, outermost, topName, indentLevel);
    }
  }

  /**
   Given an object, try to make an XML manifestation of it, and return the new
      manifestation to the caller.
   @param java.lang.Object inObject The object who XML manifestation is being built
   @param XMLInfo inInfo The optional XMLInfo associated with <code>inObject</code>
      @param String inName The optional name to use for <code>inObject</code>
   @param int indentLevel number of tabs to place in front of the emitted XML
      @param inObject
      @param inInfo
      @param inName
      @param indentLevel
      @return String containing the XML manifestation of inObject
      @throws org.quintet.monk.XMLInfoException
      @throw XMLInfoException
      @roseuid 425A86D3002D
   */
  public static String createXML(java.lang.Object inObject, XMLInfo inInfo,
                                 String inName, int indentLevel) throws
      XMLInfoException {
    if (inObject == null) {
      return null;
    }
    try {
      //see if inObject is a map.  If it is, then walk over every map entry
      //see if inObject is a collection.  If so, then call createXML for each element therein
      if (inInfo.isCollection() && (isCollection(inObject) || isArray(inObject) || isMap(inObject))) {
        Iterator memberIter = null;
        if (isArray(inObject)) {
          memberIter = ( (Collection) Arrays.asList( (Object[]) inObject)).
              iterator();
        }
        else if(isCollection(inObject)){
          memberIter = ( (Collection) inObject).iterator();

        }else{
          //it must be a map, eh?
          memberIter = ((java.util.Map)inObject).entrySet().iterator();
        }
        StringBuffer collAnswer = new StringBuffer();
        //inName has to be non-null for this, since we cannot
        //conjure up a name for this entire collection.
        if (inName == null || inName.length() == 0) {
          throw new XMLInfoException(
              "cannot process a collection/array with a null name");
        }

        //only emit the tag if inObject has at least one element or attribute
        boolean startTagEmitted = false;
        if((memberIter != null && memberIter.hasNext())){
          collAnswer.append(tabs(indentLevel) + "<" + inName + ">\n");
          startTagEmitted = true;
        }
        while (memberIter.hasNext()) {
          Object currObj = memberIter.next();

          String currAnswer = XMLFactory.createXML(currObj, indentLevel + 1);
          if (currAnswer != null) {
            collAnswer.append(currAnswer + "\n");
          }
        }
        if(startTagEmitted){
          collAnswer.append("</" + inName + ">");
        }
        String allAnswer = collAnswer.toString();
        return allAnswer;
      }

      //try to find a XMLInfo object for inObject.
      //XMLIntrospector returns a default XMLInfo
      //in cases where the developer didn't craft one by hand.
      XMLInfo info = (inInfo == null ? XMLIntrospector.getXMLInfo(inObject.getClass()) :
                      inInfo);
      if (info == null) {
        return null;
      }
      StringBuffer answer = new StringBuffer();
      String currName = (inName == null ? info.getElementName() : inName);
      answer.append(tabs(indentLevel) + "<" + inName + " ");
      //walk through each Attribute, putting out their values
      Collection attributes = info.getAttributes();
      if (attributes.size() > 0) {
        answer.append("\n");
      }
      for (Iterator attrIter = attributes.iterator(); attrIter.hasNext(); ) {
        PropertyDescriptor currAttr = (PropertyDescriptor) attrIter.next();
        String attrName = currAttr.getName();
        Method getter = currAttr.getReadMethod();
        if(getter == null){//cannot read this property
          String err = "Unable to read property " + attrName + " -- no getter method";
          log.warning(err);
          //throw new XMLInfoException(err);
        }
        Object attrValue = null;
        if(null == getter){
          attrValue = "[null]";
        }else{
          attrValue = getter.invoke(inObject, (Object[])null);
        }
        if (attrValue != null) {
          String attrStr = attrValue.toString();
          answer.append(tabs(indentLevel + 1) + attrName + " = " + "\"" +
                        attrStr + "\"" );
          if(attrIter.hasNext()){
            answer.append("\n");
          }
        }
      }

      //if there are no subordinate elements then emit the shorthand (/> terminator
      if(info.getElements() == null || info.getElements().size() == 0){
        answer.append("/>\n");
      }else{
        answer.append(">\n");
      }
      //walk through each Element, process them via XMLFactory recursively
      //Each element's XML Name is taken from its PropertyDescriptor
      Collection elements = info.getElements();
      for (Iterator elemIter = elements.iterator(); elemIter.hasNext(); ) {
        PropertyDescriptor currElem = (PropertyDescriptor) elemIter.next();
        String childElemName = currElem.getName();
        Method currElemGetter = currElem.getReadMethod();
        if(currElemGetter == null){
          String warn = "unable to get property named " + currElem.getName() + " -- no getter method";
          log.warning(warn);
          continue;
        }
        Object currElemObj = currElemGetter.invoke(inObject, (Object[])null);
        if ( currElemObj != null ) {
        		XMLInfo currElemInfo = XMLIntrospector.getXMLInfo(currElemObj.getClass());
        		String elemStr = XMLFactory.createXML(currElemObj, currElemInfo, childElemName,
                                              indentLevel + 1);
	        if (elemStr != null) {
	          answer.append(elemStr);
	        }
        }
      }

      //answer.append("\n" + tabs(indentLevel) + "</" + info.getElementName() + ">\n");
      //if subordinates were emitted then emit the terminator now
      if(info.getElements() != null && info.getElements().size() > 0)
        answer.append("\n" + tabs(indentLevel) + "</" + inName + ">\n");
      else
        answer.append("\n");
      String theAnswer = answer.toString();
      return theAnswer;
    }
    catch (Exception e) {
      e.printStackTrace();
      return null;
    }
  }

  /**
      @param inObject
      @return boolean
      @roseuid 425A86D300A5
   */
  private static boolean isCollection(Object inObject) {
    if (inObject == null) {
      return false;
    }
    return (inObject instanceof java.util.Collection);
  }

  /**
      @param inObject
      @return boolean
      @roseuid 425A86D300AF
   */
  private static boolean isMap(Object inObject) {
    if (inObject == null) {
      return false;
    }
    return (inObject instanceof java.util.Map);
  }

  /**
      @param inObject
      @return boolean
      @roseuid 425A86D300BA
   */
  private static boolean isArray(Object inObject) {
    if (inObject == null) {
      return false;
    }
    Class itsClass = inObject.getClass();
    return itsClass.isArray();
  }

  /**
      Emit the designated number of tabs
      @param int howMany The number of tabs to emit
      @param howMany
      @return String containing the prescribed number of tabs
      @roseuid 425A86D300C4
   */
  private static String tabs(int howMany) {
    while (howMany > wholeLottaTabs.length()) {
      wholeLottaTabs = wholeLottaTabs + wholeLottaTabs;
    }
    return wholeLottaTabs.substring(0, howMany);
  }
}
